#
# The mkisofs and cdrecord commands interface.
#
# Copyright (C) 1999 Steve Ratcliffe
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#

"""
	This file interfaces to the mkisofs and the cdrecord commands.

	Build up the command strings.
	Execute the commands.
	Wait for command completion
	Get command output
	Extract percentage complete information.
	Kill process on cancel
"""

from os import *
import posixfile
import string
import time

Procs = {}

# Run a command and make the output available in a non-blocking manner.
# This is much like popen3, although the output there is not specifically
# non-blocking
#
class Command:
	def __init__(self, argv):
		self.bufs = []
		self.buf = ''
		self.startTime = time.time()
		rfd, wfd = pipe()	# For stdout

		# It appears that the only way to make a non blocking file
		pf = posixfile.fileopen(fdopen(rfd))
		pf.flags('n')
		self.rfile = pf

		pid = fork()
		if pid == 0:
			# Child
			close(rfd)
			dup2(wfd, 1)
			dup2(wfd, 2)

			# Make this a new process group as cd forks into two
			# and so we need to kill a process group and not just
			# the process we started.
			setsid()

			# Print out here so it will appear in the text window.
			print "\n\nCommand:", string.join(argv)
			try:
				execvp(argv[0], argv)
			except:
				execvp('/bin/false', ['false'])

				# Unfortunately this does not appear to work...
				# which is why I have the /bin/false
				_exit(1)

		# In parent
		close(wfd)
		Procs[pid] = ''	# Create an entry in the dictionary for it
		self.pid = pid
		self.status = -1

	def readline(self):
		stuff = ''
		try:
			stuff = self.rfile.read()
			if stuff:
				# Get rid of any CR and turn them into NL
				stuff = string.replace(stuff, '\r', '\n')
				if stuff[0] == '\b':
					# Groan -- cdrecord uses backspaces to do the countdown
					# deal with it by duplicating the line and appending
					# the backspaced line.
					r = 0
					for c in stuff:
						if c != '\b':
							break
						r = r+1
					self.bufs.append('\n')	# Force out previous line
					self.bufs.append(self.buf[:-r] + stuff[r:])
				else:
					self.bufs.append(stuff)
		except IOError, msg:
			pass

		self._joinbufs()
		try:
			r = string.index(self.buf, '\n')
		except ValueError:
			pass
		else:
			ret = self.buf[:r+1]
			self.buf = self.buf[r+1:]
			return ret

		return ''

	def _joinbufs(self):
		if self.bufs:
			for b in self.bufs:
				self.buf = self.buf + b
			self.bufs = []

	# Check to see if the command has exited
	def has_exited(self):
		if self.status >= 0:
			return 1	# Already finished

		try:
			pid, status = waitpid(self.pid, WNOHANG)
		except error:
			status = 1000
			return 1	# Probably not even running

		if pid == self.pid:
			self.status = status
			try: del Procs[self.pid]
			except KeyError: pass
			return 1

		# Still running
		return 0

	# Kill the process group created above
	def kill(self):
		try: kill(-self.pid, 2)
		except OSError, msg: print "DEBUG: could not zap", msg
		time.sleep(0.2)
		try: kill(-self.pid, 15)
		except OSError: pass
		try: del Procs[self.pid]
		except KeyError, msg: print "DEBUG: could not delete pid", msg

	# Wait for the command
	def wait(self):
		while self.status < 0:
			try:
				pid, status = waitpid(self.pid, 0)
			except OSError:
				break
			if pid == self.pid:
				self.status = status

	def __del__(self):
		#print "COMMAND DELETED", self.pid
		pass

def killall():
	for pid in Procs.keys():
		try: kill(pid, 9)
		except OSError: pass
	global Procs
	Procs = {}

# For testing only
if __name__ == '__main__':
	# Test program
	c = Command(['mkisofs', '-v', '-o', '/tmp/foo.fs', '/home/steve/bak'])
	while 1:
		b = c.readline()
		if b:
			print "OUT", b
		elif c.has_exited():
			while 1:
				b = c.readline()
				if b is None or b == '': break
				print "OUT2", b
			break
