#
# Configuration file handling
#
# Copyright (C) 1999 Steve Ratcliffe
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#

import os, stat, string, glob
import ConfigParser
import cdbak

# A default configuration
#
default_config = {
	"allfiles": """no""",
	"application_id": """Backup""",
	"deep_directory_relocation": """no""",
	"exclude_path": """""",
	"exclude_pattern": """""",
	"follow_symlinks": """no""",
	"output": """/usr/tmp/backup.raw""",	# yea no chance
	"preparer_id": """Steve Ratcliffe <steve@parabola.demon.co.uk>""",
	"publisher_id": """Steve Ratcliffe <steve@parabola.demon.co.uk>""",
	"rockridge": """yes""",
	"use_joliet": """no""",
}


# Get config information
def get_config():
	"""Get configuration information
	"""
	conf = ConfigParser.ConfigParser(default_config)
	conf_files = get_conf_files()
	conf.read(conf_files)

	return conf

def get_file_lists(conf):
	"""
	Return a list of the configured file list options
	
	It is in the form:
	   ('listname', [('path-on-disk', 'path-to-be-placed-on-CD'), ...]), ...
	"""

	conflist = []
	for s in conf.sections():
		listname = s
		for v in conf.options(s):
			if v == 'backup':
				files = parse_backup_spec(conf.get(s, v))
				break
		
		conflist = conflist + [(listname, files)]

	return conflist


def parse_backup_spec(spec):
	list = []

	lines1 = string.split(spec, '\n')

	# Lines ending in a comma should be joined with the next line
	lines = []
	i = 0
	while i < len(lines1):
		line = string.strip(lines1[i])

		# If the line ends in a comma then join the next one to it.
		#
		while line[-1:] == ',':
			try:
				line = line + lines1[i+1]
				i = i+1
			except IndexError:
				break

		# Add line in
		lines.append(line)
		i = i+1

	for line in lines:
		line = string.strip(line)
		tok = string.split(line)

		if len(tok) < 1:
			continue  # A blank line

		if tok[0] == 'dir' or tok[0] == 'file':
			item = parse_dir(tok)
		elif tok[0] == 'file':
			item = parse_file(tok)
		else:
			raise cdbak.error, "Bad command in configuration file: " + line

		list = list + item

	return list

def parse_dir(t):
	l = []

	# Join it back together, without first line
	str = string.join(t[1:])

	# Split by comma separated fields
	fields = string.split(str, ',')
	if len(fields) == 0:
		raise cdbak.error, "Program error"   # This can't happen

	# The first field is either a single directory or a pair of directories.
	dirs = fields[0]
	dirspec = string.split(dirs)
	base = add_slash(dirspec[0])

	# If no alternate directory was given then we use the base name
	if len(dirspec) == 1:
		todir = base
	else:
		todir = add_slash(dirspec[1])

	#
	# Now deal with the rest of the fields
	#
	exceptlist = []
	for f in fields[1:]:
		tok = string.split(f)
		if len(tok) == 0: continue

		if tok[0] == 'except':
			# These are files/directories to be excluded
			for word in tok[1:]:
				exceptlist.append(base + word + '/*')
		else:
			raise cdbak.error, "Incorrect sub-option in configuration file: "\
				+tok[0]

	l.append((base, todir, exceptlist))

	return l

def parse_file(t):
	return [(t[0], 'WARNING: Not implemented yet')]


def get_conf_files():
	"""Get a list of places to try for configuration information.
	"""
	list = []

	#
	# This list must be ordered from least specific to most as later
	# definitions will override earlier ones.  The files also get merged.
	#
	list.append('/etc/cdbak.conf')

	try:
		f = os.environ['HOME'] + '/' + '.cdbak.conf'
		list.append(f)
	except KeyError: pass

	try:
		f = os.environ['CDBAK']
		list.append(f)
	except KeyError: pass

	return list

# Add a slash to the end of the path name if not already there.
def add_slash(name):
	if len(name) == 0:
		return '/'
	elif name[-1] != '/':
		return name + '/'
	else:
		return name
