#
# A control class for dealing with callbacks from the gui front end.
#
# Copyright (C) 1999 Steve Ratcliffe
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#

import string
import time
import cmds
import gtk
import gui


"""
	This is the main logic control for the program.  It interfaces
	between the gui interface and the backend commands.
	All control of the interface is done from here.
"""
#
class Callbacks:
	def __init__(self):
		self.Command = None
		self.Page = 'make'
		self.completionProgress = []
		self.okDialog = gui.OKdialog()

	def set_frontend(self, w):
		self.Interface = w

	def set_cmd(self, cmds):
		self.Command = cmds
		self.percentComplete = 0

	def set_conf(self, conf):
		self.Config = conf

	def onNextPrev(self, (name, num), inc):
		"""
		Called when the Next/Prev button is clicked.

		Should do whatever operation is required and cause the relevant
		page of the gui to be displayed.
		"""

		self.doPageChange(inc)

	def onCancel(self, what):
		""" Called when the cancel button is clicked
		"""
		if self.Command is not None:
			self.Command.kill()
			self.Command.wait()
			self.okDialog.show_message("Command was canceled",
				self.doPageChange, -1)
			self.Command = None

	def profileSelect(self, name, flist):
		""" Called when a profile is selected by the user.
		The selected profile name is passed in.
		"""

		if name[:2] == '--':
			return

		self.Profile = name
		self.FileList = flist
		self.Interface.allow_step()

	# This command runs in the idle loop, and updates the status
	# window with any output from the commands.
	def monitor_cmd(self, prog=None):
		if prog is None:
			prog = self.currentProgress
		if self.Command is None:
			return 0
		while 1:
			line = self.Command.readline()
			# If the line is null then we check to see if the command
			# has finished
			if line == '':

				self.set_percent(prog)	# Interpolate percentage progress!

				if self.Command.status >= 0:
					break	# Already waited for it
				elif self.Command.has_exited():
					# This will set Status so we won't get here again
					continue
				else:
					break
			else:
				if line[-1] != '\n':
					line = line + '\n'
				self.Interface.StatusText.insert_defaults(line)

				# Look for progress indications
				#  xx.xx% done, estimate finish...
				#  23 of 123MB ...
				#
				if string.find(line, 'done, estimate finish') >= 0:
					f = string.atof(line[:string.index(line, '%')])
					f = f/100
					if f > 1:
						f = 1.0
					self.set_percent(prog, f) 
				elif string.find(line, 'MB written') > 0:
					# Should be: 'Track 01:  9 of 123 MB written ...
					toks = string.split(line)
					f = string.atof(toks[2]) / string.atof(toks[4])
					if f > 1:
						f = 1.0
					self.set_percent(prog, f) 

		if self.Command.status >= 0:

			if self.Command.status == 0:
				# Command successful..
				self.doPageChange(1)
			else:
				self.okDialog.show_message("Command failed",
					self.doPageChange, -1)

			self.Command = None
			self.completionProgress = []
			return 0  # We sould not be called again, command is finished

		return 1

	#
	# Set the percentage progress bar.  We use indications from the
	# output about how the progress is going.  Inbetween we interpolate
	# to get a smooth progress indication.
	#
	def set_percent(self, prog, percent = None):
		curtime = time.time() - self.Command.startTime
		if percent is None:
			# Interpolate given the current time and previous
			# progress.
			try:
				l = len(self.completionProgress)
				if l > 1:
					(t1, p1) = self.completionProgress[l-1]
					(t2, p2) = self.completionProgress[0]
					rate = (p1-p2)/(t1-t2)
				elif l == 1:
					(t1, p1) = self.completionProgress[0]
					rate = p1/t1
				else:
					raise ValueError

				nt = curtime - t1	# Time since last real information
				np = rate * nt		# Estimated extra progress
				p = p1 + np		# Estimated total progress
				#print "t1=%f, cur=%f, rate=%f" % (t1, curtime, rate)
				#print "nt=%f, np=%f, p=%f" % (nt, np, p)
				if p > 1:
					p = 1.0
				prog.set_percentage(p)
			except ValueError:
				pass  # Thats OK just no info yet.
		else:
			if percent > 1:
				percent = 1.0

			self.completionProgress.append((curtime, percent))
			prog.set_percentage(percent)


	# Do the transition from one state to another.
	def doPageChange(self, inc):
		for li in States:
			if li[CURRENT] == self.Page:
				if inc > 0:
					npage = li[NEXT_PAGE]
					li[NEXT_ACTION](self, npage)
				else:
					npage = li[PREV_PAGE]
					li[PREV_ACTION](self, npage)
				break
		else:
			print "ERROR: on unknown page"
			raise SystemExit
		
		self.Page = npage


def domake(c, page):
	proc = cmds.mkisofs(c.Config, c.Profile, c.FileList)
	c.set_cmd(proc)
	cdbak_add_timeout(c, 100, c.monitor_cmd, c.Interface.progressPrepare)
	steppage(c, page, allow=0)

def finmake(c, page):
	steppage(c, page)

def dowrite(c, page):
	proc = cmds.cdrecord(c.Config, c.Profile)
	c.set_cmd(proc)
	cdbak_add_timeout(c, 100, c.monitor_cmd, c.Interface.progressWrite)
	steppage(c, page, allow=0)

# In gtk the timeout_add function takes a third argument.  pygtk does
# not.  It is possible that pygtk will be fixed in the future though.
# This routine works around this and should work in both cases.
def cdbak_add_timeout(c, interval, func, arg):
	try:
		gtk.timeout_add(interval, func, arg)
	except TypeError:
		gtk.timeout_add(interval, func)
		c.currentProgress = arg

def finwrite(c, page):
	steppage(c, page)

def steppage(c, name, allow=1):
	c.Interface.set_page(name)
	c.Interface.allow_step(allow)


# State transition table.
[
	CURRENT,		# Current page
	NEXT_ACTION,	# Action when Next is clicked
	NEXT_PAGE,		# The next page to go to
	PREV_ACTION,	# Action when Prev is clicked
	PREV_PAGE,		# The page to go to on a Prev
] = range(5)

States = [
	['make',
		domake, 'make_progress',
		steppage, 'make',
	],
	['make_progress',
		finmake, 'write',
		steppage, 'make',
	],
	['write',
		dowrite, 'write_progress',
		steppage, 'make',
	],
	['write_progress',
		finwrite, 'finished',
		steppage, 'make',
	],
	['finished',
		steppage, 'make',
		steppage, 'make',
	],
]
